@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011 Douglas Gilbert.
+ * Copyright (c) 2009-2014 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -24,10 +24,11 @@
 /* A utility program originally written for the Linux OS SCSI subsystem.
  *
  *
- * This program issues the SCSI GET LBA STATUS command to the given SCSI device.
+ * This program issues the SCSI GET LBA STATUS command to the given SCSI
+ * device.
  */
 
-static char * version_str = "1.03 20110204";    /* sbc2r22 */
+static const char * version_str = "1.06 20140515";    /* sbc2r29 */
 
 #define MAX_GLBAS_BUFF_LEN (1024 * 1024)
 #define DEF_GLBAS_BUFF_LEN 24
@@ -43,6 +44,7 @@ static struct option long_options[] = {
         {"lba", required_argument, 0, 'l'},
         {"maxlen", required_argument, 0, 'm'},
         {"raw", no_argument, 0, 'r'},
+        {"readonly", no_argument, 0, 'R'},
         {"verbose", no_argument, 0, 'v'},
         {"version", no_argument, 0, 'V'},
         {0, 0, 0, 0},
@@ -53,9 +55,9 @@ usage()
 {
     fprintf(stderr, "Usage: "
             "sg_get_lba_status  [--brief] [--help] [--hex] [--lba=LBA]\n"
-            "                          [--maxlen=LEN] [--raw] [--verbose] "
-            "[--version]\n"
-            "                          DEVICE\n"
+            "                          [--maxlen=LEN] [--raw] [--readonly] "
+            "[--verbose]\n"
+            "                          [--version] DEVICE\n"
             "  where:\n"
             "    --brief|-b        a descriptor per line: "
             "<lba_hex blocks_hex p_status>\n"
@@ -71,6 +73,7 @@ usage()
             DEF_GLBAS_BUFF_LEN );
     fprintf(stderr,
             "    --raw|-r          output in binary\n"
+            "    --readonly|-R     open DEVICE read-only (def: read-write)\n"
             "    --verbose|-v      increase verbosity\n"
             "    --version|-V      print version string and exit\n\n"
             "Performs a SCSI GET LBA STATUS command (SBC-3)\n"
@@ -131,6 +134,7 @@ main(int argc, char * argv[])
     uint32_t d_blocks = 0;
     int maxlen = DEF_GLBAS_BUFF_LEN;
     int do_raw = 0;
+    int o_readonly = 0;
     int verbose = 0;
     const char * device_name = NULL;
     const unsigned char * ucp;
@@ -139,7 +143,7 @@ main(int argc, char * argv[])
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "bhHl:m:rvV", long_options,
+        c = getopt_long(argc, argv, "bhHl:m:rRvV", long_options,
                         &option_index);
         if (c == -1)
             break;
@@ -174,6 +178,9 @@ main(int argc, char * argv[])
         case 'r':
             ++do_raw;
             break;
+        case 'R':
+            ++o_readonly;
+            break;
         case 'v':
             ++verbose;
             break;
@@ -220,7 +227,7 @@ main(int argc, char * argv[])
         }
     }
 
-    sg_fd = sg_cmds_open_device(device_name, 0 /* rw */, verbose);
+    sg_fd = sg_cmds_open_device(device_name, o_readonly, verbose);
     if (sg_fd < 0) {
         fprintf(stderr, "open error: %s: %s\n", device_name,
                 safe_strerror(-sg_fd));
@@ -232,9 +239,11 @@ main(int argc, char * argv[])
                                verbose);
     ret = res;
     if (0 == res) {
+        /* in sbc3r25 offset for calculating the 'parameter data length'
+         * (rlen variable below) was reduced from 8 to 4. */
         if (maxlen >= 4)
             rlen = (glbasBuffp[0] << 24) + (glbasBuffp[1] << 16) +
-                   (glbasBuffp[2] << 8) + glbasBuffp[3] + 8;
+                   (glbasBuffp[2] << 8) + glbasBuffp[3] + 4;
         else
             rlen = maxlen;
         k = (rlen > maxlen) ? maxlen : rlen;
@@ -260,7 +269,7 @@ main(int argc, char * argv[])
         }
         if (rlen > maxlen)
             rlen = maxlen;
-        
+
         if (do_brief > 1) {
             if (rlen < 24) {
                 fprintf(stderr, "Need maxlen and response length to "
@@ -333,14 +342,13 @@ main(int argc, char * argv[])
                     "found\n");
     } else if (SG_LIB_CAT_INVALID_OP == res)
         fprintf(stderr, "Get LBA Status command not supported\n");
-    else if (SG_LIB_CAT_ABORTED_COMMAND == res)
-        fprintf(stderr, "Get LBA Status, aborted command\n");
     else if (SG_LIB_CAT_ILLEGAL_REQ == res)
-        fprintf(stderr, "Get LBA Status command has bad field in cdb\n");
+        fprintf(stderr, "Get LBA Status command: bad field in cdb\n");
     else {
-        fprintf(stderr, "Get LBA Status command failed\n");
-        if (0 == verbose)
-            fprintf(stderr, "    try '-v' option for more information\n");
+        char b[80];
+
+        sg_get_category_sense_str(res, sizeof(b), b, verbose);
+        fprintf(stderr, "Get LBA Status command: %s\n", b);
     }
 
 the_end:
