@@ -1,30 +1,8 @@
 /*
- * Copyright (c) 2005-2009 Douglas Gilbert.
+ * Copyright (c) 2005-2014 Douglas Gilbert.
  * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
+ * Use of this source code is governed by a BSD-style
+ * license that can be found in the BSD_LICENSE file.
  */
 
 #include <unistd.h>
@@ -54,9 +32,7 @@
  * vendor specific data is written.
  */
 
-static char * version_str = "1.12 20090611";
-
-#define ME "sg_reassign: "
+static const char * version_str = "1.16 20140517";
 
 #define DEF_DEFECT_LIST_FORMAT 4        /* bytes from index */
 
@@ -119,7 +95,7 @@ usage()
 
 /* Trying to decode multipliers as sg_get_llnum() [in sg_libs] does would
  * only confuse things here, so use this local trimmed version */
-int64_t
+static int64_t
 get_llnum(const char * buf)
 {
     int res, len;
@@ -168,12 +144,13 @@ build_lba_arr(const char * inp, uint64_t * lba_arr,
     if (0 == in_len)
         *lba_arr_len = 0;
     if ('-' == inp[0]) {        /* read from stdin */
-        char line[512];
+        char line[1024];
         int off = 0;
 
         for (j = 0; j < 512; ++j) {
             if (NULL == fgets(line, sizeof(line), stdin))
                 break;
+            // could improve with carry_over logic if sizeof(line) too small
             in_len = strlen(line);
             if (in_len > 0) {
                 if ('\n' == line[in_len - 1]) {
@@ -181,7 +158,7 @@ build_lba_arr(const char * inp, uint64_t * lba_arr,
                     line[in_len] = '\0';
                 }
             }
-            if (0 == in_len)
+            if (in_len < 1)
                 continue;
             lcp = line;
             m = strspn(lcp, " \t");
@@ -236,8 +213,8 @@ build_lba_arr(const char * inp, uint64_t * lba_arr,
             ll = get_llnum(lcp);
             if (-1 != ll) {
                 lba_arr[k] = (uint64_t)ll;
-                cp = strchr(lcp, ',');
-                c2p = strchr(lcp, ' ');
+                cp = (char *)strchr(lcp, ',');
+                c2p = (char *)strchr(lcp, ' ');
                 if (NULL == cp)
                     cp = c2p;
                 if (NULL == cp)
@@ -277,6 +254,7 @@ main(int argc, char * argv[])
     const char * device_name = NULL;
     uint64_t addr_arr[MAX_NUM_ADDR];
     unsigned char param_arr[4 + (MAX_NUM_ADDR * 8)];
+    char b[80];
     int param_len = 4;
     int ret = 0;
 
@@ -336,7 +314,7 @@ main(int argc, char * argv[])
             ++verbose;
             break;
         case 'V':
-            fprintf(stderr, ME "version: %s\n", version_str);
+            fprintf(stderr, "version: %s\n", version_str);
             return 0;
         default:
             fprintf(stderr, "unrecognised option code 0x%x ??\n", c);
@@ -415,7 +393,7 @@ main(int argc, char * argv[])
 
     sg_fd = sg_cmds_open_device(device_name, 0 /* rw */, verbose);
     if (sg_fd < 0) {
-        fprintf(stderr, ME "open error: %s: %s\n", device_name,
+        fprintf(stderr, "open error: %s: %s\n", device_name,
                 safe_strerror(-sg_fd));
         return SG_LIB_FILE_ERROR;
     }
@@ -433,23 +411,9 @@ main(int argc, char * argv[])
         res = sg_ll_reassign_blocks(sg_fd, eight, longlist, param_arr,
                                     param_len, 1, verbose);
         ret = res;
-        if (SG_LIB_CAT_NOT_READY == res) {
-            fprintf(stderr, "REASSIGN BLOCKS failed, device not ready\n");
-            goto err_out;
-        } else if (SG_LIB_CAT_UNIT_ATTENTION == res) {
-            fprintf(stderr, "REASSIGN BLOCKS, unit attention\n");
-            goto err_out;
-        } else if (SG_LIB_CAT_ABORTED_COMMAND == res) {
-            fprintf(stderr, "REASSIGN BLOCKS, aborted command\n");
-            goto err_out;
-        } else if (SG_LIB_CAT_INVALID_OP == res) {
-            fprintf(stderr, "REASSIGN BLOCKS not supported\n");
-            goto err_out;
-        } else if (SG_LIB_CAT_ILLEGAL_REQ == res) {
-            fprintf(stderr, "bad field in REASSIGN BLOCKS cdb\n");
-            goto err_out;
-        } else if (0 != res) {
-            fprintf(stderr, "REASSIGN BLOCKS failed\n");
+        if (res) {
+            sg_get_category_sense_str(res, sizeof(b), b, verbose);
+            fprintf(stderr, "REASSIGN BLOCKS: %s\n", b);
             goto err_out;
         }
     } else /* if (grown || primary) */ {
@@ -463,18 +427,9 @@ main(int argc, char * argv[])
         res = sg_ll_read_defect10(sg_fd, primary, grown, dl_format,
                                   param_arr, param_len, 0, verbose);
         ret = res;
-        if (SG_LIB_CAT_NOT_READY == res) {
-            fprintf(stderr, "READ DEFECT DATA (10) failed, device not "
-                    "ready\n");
-            goto err_out;
-        } else if (SG_LIB_CAT_INVALID_OP == res) {
-            fprintf(stderr, "READ DEFECT DATA (10) not supported\n");
-            goto err_out;
-        } else if (SG_LIB_CAT_ILLEGAL_REQ == res) {
-            fprintf(stderr, "bad field in READ DEFECT DATA (10) cdb\n");
-            goto err_out;
-        } else if (0 != res) {
-            fprintf(stderr, "READ DEFECT DATA (10) failed\n");
+        if (res) {
+            sg_get_category_sense_str(res, sizeof(b), b, verbose);
+            fprintf(stderr, "READ DEFECT DATA(10): %s\n", b);
             goto err_out;
         }
         if (do_hex) {
