@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2011 Luben Tuikov and Douglas Gilbert.
+ * Copyright (c) 2006-2014 Luben Tuikov and Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -9,6 +9,7 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <ctype.h>
 #include <string.h>
 #include <getopt.h>
@@ -25,7 +26,7 @@
  * This utility issues the SCSI WRITE BUFFER command to the given device.
  */
 
-static char * version_str = "1.10 20110825";    /* spc4r32 */
+static const char * version_str = "1.15 20140518";    /* spc4r37 */
 
 #define ME "sg_write_buffer: "
 #define DEF_XFER_LEN (8 * 1024 * 1024)
@@ -34,55 +35,83 @@ static char * version_str = "1.10 20110825";    /* spc4r32 */
 #define WRITE_BUFFER_CMD 0x3b
 #define WRITE_BUFFER_CMDLEN 10
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
-#define DEF_PT_TIMEOUT 60       /* 60 seconds */
+#define DEF_PT_TIMEOUT 120       /* 120 seconds, 2 minutes */
 
 static struct option long_options[] = {
-        {"help", 0, 0, 'h'},
-        {"id", 1, 0, 'i'},
-        {"in", 1, 0, 'I'},
-        {"length", 1, 0, 'l'},
-        {"mode", 1, 0, 'm'},
-        {"offset", 1, 0, 'o'},
-        {"raw", 0, 0, 'r'},
-        {"skip", 1, 0, 's'},
-        {"specific", 1, 0, 'S'},
-        {"verbose", 0, 0, 'v'},
-        {"version", 0, 0, 'V'},
+        {"bpw", required_argument, 0, 'b'},
+        {"help", no_argument, 0, 'h'},
+        {"id", required_argument, 0, 'i'},
+        {"in", required_argument, 0, 'I'},
+        {"length", required_argument, 0, 'l'},
+        {"mode", required_argument, 0, 'm'},
+        {"offset", required_argument, 0, 'o'},
+        {"raw", no_argument, 0, 'r'},
+        {"skip", required_argument, 0, 's'},
+        {"specific", required_argument, 0, 'S'},
+        {"verbose", no_argument, 0, 'v'},
+        {"version", no_argument, 0, 'V'},
         {0, 0, 0, 0},
 };
 
+#ifdef __GNUC__
+static int pr2serr(const char * fmt, ...)
+        __attribute__ ((format (printf, 1, 2)));
+#else
+static int pr2serr(const char * fmt, ...);
+#endif
+
+
+static int
+pr2serr(const char * fmt, ...)
+{
+    va_list args;
+    int n;
+
+    va_start(args, fmt);
+    n = vfprintf(stderr, fmt, args);
+    va_end(args);
+    return n;
+}
+
 
 static void
 usage()
 {
-    fprintf(stderr, "Usage: "
-          "sg_write_buffer [--help] [--id=ID] [--in=FILE] "
-          "[--length=LEN]\n"
-          "                       [--mode=MO] [--offset=OFF] [--raw] "
-          "[--skip=SKIP]\n"
-          "                       [--specific=MS] [--verbose] [--version] "
-          "DEVICE\n"
-          "  where:\n"
-          "    --help|-h              print out usage message then exit\n"
-          "    --id=ID|-i ID          buffer identifier (0 (default) to "
-          "255)\n"
-          "    --in=FILE|-I FILE      read from FILE ('-I -' read "
-          "from stdin)\n"
-          "    --length=LEN|-l LEN    length in bytes to write; may be "
-          "deduced from FILE\n"
-          "    --mode=MO|-m MO        write buffer mode, MO is number or "
-          "acronym (def: 0)\n"
-          "    --off=OFF|-o OFF       buffer offset (unit: bytes, def: 0)\n"
-          "    --raw|-r               read from stdin (same as '-I -')\n"
-          "    --skip=SKIP|-s SKIP    bytes in file FILE to skip before "
-          "reading\n"
-          "    --specific=MS|-S MS    mode specific value; 3 bit field "
-          "(0 to 7)\n"
-          "    --verbose|-v           increase verbosity\n"
-          "    --version|-V           print version string and exit\n\n"
-          "  Numbers given in options are decimal unless they have a "
-          "hex indicator\n"
-          "Performs a SCSI WRITE BUFFER command\n"
+    pr2serr("Usage: "
+            "sg_write_buffer [--bpw=CS] [--help] [--id=ID] [--in=FILE]\n"
+            "                       [--length=LEN] [--mode=MO] "
+            "[--offset=OFF] [--raw]\n"
+            "                       [--skip=SKIP] [--specific=MS] "
+            "[--verbose] [--version]\n"
+            "                       DEVICE\n"
+            "  where:\n"
+            "    --bpw=CS|-b CS         CS is chunk size: bytes per write "
+            "buffer\n"
+            "                           command (def: 0 -> as many as "
+            "possible)\n"
+            "    --help|-h              print out usage message then exit\n"
+            "    --id=ID|-i ID          buffer identifier (0 (default) to "
+            "255)\n"
+            "    --in=FILE|-I FILE      read from FILE ('-I -' read "
+            "from stdin)\n"
+            "    --length=LEN|-l LEN    length in bytes to write; may be "
+            "deduced from\n"
+            "                           FILE\n"
+            "    --mode=MO|-m MO        write buffer mode, MO is number or "
+            "acronym\n"
+            "                           (def: 0 -> 'combined header and "
+            "data' (obs))\n"
+            "    --off=OFF|-o OFF       buffer offset (unit: bytes, def: 0)\n"
+            "    --raw|-r               read from stdin (same as '-I -')\n"
+            "    --skip=SKIP|-s SKIP    bytes in file FILE to skip before "
+            "reading\n"
+            "    --specific=MS|-S MS    mode specific value; 3 bit field "
+            "(0 to 7)\n"
+            "    --verbose|-v           increase verbosity\n"
+            "    --version|-V           print version string and exit\n\n"
+            "Performs one or more SCSI WRITE BUFFER commands. Use '-m xxx' "
+            "to list\navailable modes. Numbers given in options are decimal "
+            "unless they have\na hex indicator.\n"
           );
 
 }
@@ -104,11 +133,12 @@ usage()
 
 
 static struct mode_s {
-        char *mode_string;
+        const char *mode_string;
         int   mode;
-        char *comment;
+        const char *comment;
 } modes[] = {
-        { "hd",         MODE_HEADER_DATA, "combined header and data"},
+        { "hd",         MODE_HEADER_DATA, "combined header and data "
+                "(obsolete)"},
         { "vendor",     MODE_VENDOR,    "vendor specific"},
         { "data",       MODE_DATA,      "data"},
         { "dmc",        MODE_DNLD_MC,   "download microcode and activate"},
@@ -127,10 +157,11 @@ static struct mode_s {
         { "activate_mc", MODE_ACTIVATE_MC,
                 "activate deferred microcode"},
         { "en_ex",      MODE_EN_EX_ECHO, "enable expander communications "
-                "protocol and echo\n\t\t\t\tbuffer"},
+                "protocol and echo\n\t\t\t\tbuffer (obsolete)"},
         { "dis_ex",     MODE_DIS_EX, "disable expander communications "
-                "protocol"},
-        { "deh",        MODE_DNLD_ERR_HISTORY, "download error history "},
+                "protocol\n\t\t\t\t(obsolete)"},
+        { "deh",        MODE_DNLD_ERR_HISTORY, "download application client "
+                "error history "},
 };
 
 #define NUM_MODES       ((int)(sizeof(modes)/sizeof(modes[0])))
@@ -140,11 +171,11 @@ print_modes(void)
 {
     int k;
 
-    fprintf(stderr, "The modes parameter argument can be numeric "
-                "(hex or decimal)\nor symbolic:\n");
+    pr2serr("The modes parameter argument can be numeric (hex or decimal)\n"
+            "or symbolic:\n");
     for (k = 0; k < NUM_MODES; k++) {
-        fprintf(stderr, " %2d (0x%02x)  %-18s%s\n", modes[k].mode,
-                modes[k].mode, modes[k].mode_string, modes[k].comment);
+        pr2serr(" %2d (0x%02x)  %-18s%s\n", modes[k].mode, modes[k].mode,
+                modes[k].mode_string, modes[k].comment);
     }
 }
 
@@ -174,24 +205,22 @@ sg_ll_write_buffer_v2(int sg_fd, int mode, int m_specific, int buffer_id,
     wbufCmdBlk[6] = (unsigned char)((param_len >> 16) & 0xff);
     wbufCmdBlk[7] = (unsigned char)((param_len >> 8) & 0xff);
     wbufCmdBlk[8] = (unsigned char)(param_len & 0xff);
-    if (NULL == sg_warnings_strm)
-        sg_warnings_strm = stderr;
     if (verbose) {
-        fprintf(sg_warnings_strm, "    Write buffer cmd: ");
+        pr2serr("    Write buffer cmd: ");
         for (k = 0; k < WRITE_BUFFER_CMDLEN; ++k)
-            fprintf(sg_warnings_strm, "%02x ", wbufCmdBlk[k]);
-        fprintf(sg_warnings_strm, "\n");
+            pr2serr("%02x ", wbufCmdBlk[k]);
+        pr2serr("\n");
         if ((verbose > 1) && paramp && param_len) {
-            fprintf(sg_warnings_strm, "    Write buffer parameter list%s:\n",
+            pr2serr("    Write buffer parameter list%s:\n",
                     ((param_len > 256) ? " (first 256 bytes)" : ""));
-            dStrHex((const char *)paramp,
-                    ((param_len > 256) ? 256 : param_len), -1);
+            dStrHexErr((const char *)paramp,
+                       ((param_len > 256) ? 256 : param_len), -1);
         }
     }
 
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
-        fprintf(sg_warnings_strm, "write buffer: out of memory\n");
+        pr2serr("write buffer: out of memory\n");
         return -1;
     }
     set_scsi_pt_cdb(ptvp, wbufCmdBlk, sizeof(wbufCmdBlk));
@@ -204,19 +233,12 @@ sg_ll_write_buffer_v2(int sg_fd, int mode, int m_specific, int buffer_id,
         ;
     else if (-2 == ret) {
         switch (sense_cat) {
-        case SG_LIB_CAT_NOT_READY:
-        case SG_LIB_CAT_INVALID_OP:
-        case SG_LIB_CAT_ILLEGAL_REQ:
-        case SG_LIB_CAT_UNIT_ATTENTION:
-        case SG_LIB_CAT_ABORTED_COMMAND:
-            ret = sense_cat;
-            break;
         case SG_LIB_CAT_RECOVERED:
         case SG_LIB_CAT_NO_SENSE:
             ret = 0;
             break;
         default:
-            ret = -1;
+            ret = sense_cat;
             break;
         }
     } else
@@ -230,7 +252,9 @@ sg_ll_write_buffer_v2(int sg_fd, int mode, int m_specific, int buffer_id,
 int
 main(int argc, char * argv[])
 {
-    int sg_fd, infd, res, c, len, k, got_stdin;
+    int sg_fd, infd, res, c, len, k, n, got_stdin;
+    int bpw = 0;
+    int bpw_then_activate = 0;
     int do_help = 0;
     int wb_id = 0;
     int wb_len = 0;
@@ -243,18 +267,31 @@ main(int argc, char * argv[])
     const char * device_name = NULL;
     const char * file_name = NULL;
     unsigned char * dop = NULL;
+    char * cp;
     char ebuff[EBUFF_SZ];
     int ret = 0;
 
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "hi:I:l:m:o:rs:S:vV", long_options,
+        c = getopt_long(argc, argv, "b:hi:I:l:m:o:rs:S:vV", long_options,
                         &option_index);
         if (c == -1)
             break;
 
         switch (c) {
+        case 'b':
+            bpw = sg_get_num(optarg);
+            if (bpw < 0) {
+                pr2serr("argument to '--bpw' should be in a positive "
+                        "number\n");
+                return SG_LIB_SYNTAX_ERROR;
+            }
+            if ((cp = strchr(optarg, ','))) {
+                if (0 == strncmp("act", cp + 1, 3))
+                    ++bpw_then_activate;
+            }
+            break;
         case 'h':
         case '?':
             ++do_help;
@@ -262,8 +299,8 @@ main(int argc, char * argv[])
         case 'i':
             wb_id = sg_get_num(optarg);
             if ((wb_id < 0) || (wb_id > 255)) {
-                fprintf(stderr, "argument to '--id' should be in the range "
-                        "0 to 255\n");
+                pr2serr("argument to '--id' should be in the range 0 to "
+                        "255\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             break;
@@ -273,7 +310,7 @@ main(int argc, char * argv[])
         case 'l':
             wb_len = sg_get_num(optarg);
             if (wb_len < 0) {
-                fprintf(stderr, "bad argument to '--length'\n");
+                pr2serr("bad argument to '--length'\n");
                 return SG_LIB_SYNTAX_ERROR;
              }
              wb_len_given = 1;
@@ -282,8 +319,8 @@ main(int argc, char * argv[])
             if (isdigit(*optarg)) {
                 wb_mode = sg_get_num(optarg);
                 if ((wb_mode < 0) || (wb_mode > 31)) {
-                    fprintf(stderr, "argument to '--mode' should be in the "
-                            "range 0 to 31\n");
+                    pr2serr("argument to '--mode' should be in the range 0 "
+                            "to 31\n");
                     return SG_LIB_SYNTAX_ERROR;
                 }
             } else {
@@ -303,7 +340,7 @@ main(int argc, char * argv[])
         case 'o':
            wb_offset = sg_get_num(optarg);
            if (wb_offset < 0) {
-                fprintf(stderr, "bad argument to '--offset'\n");
+                pr2serr("bad argument to '--offset'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             break;
@@ -313,15 +350,14 @@ main(int argc, char * argv[])
         case 's':
            wb_skip = sg_get_num(optarg);
            if (wb_skip < 0) {
-                fprintf(stderr, "bad argument to '--skip'\n");
+                pr2serr("bad argument to '--skip'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             break;
         case 'S':
            wb_mspec = sg_get_num(optarg);
            if ((wb_mspec < 0) || (wb_mspec > 7)) {
-                fprintf(stderr, "expected argument to '--specific' to be "
-                        "0 to 7\n");
+                pr2serr("expected argument to '--specific' to be 0 to 7\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             break;
@@ -329,10 +365,10 @@ main(int argc, char * argv[])
             ++verbose;
             break;
         case 'V':
-            fprintf(stderr, ME "version: %s\n", version_str);
+            pr2serr(ME "version: %s\n", version_str);
             return 0;
         default:
-            fprintf(stderr, "unrecognised option code 0x%x ??\n", c);
+            pr2serr("unrecognised option code 0x%x ??\n", c);
             usage();
             return SG_LIB_SYNTAX_ERROR;
         }
@@ -340,7 +376,7 @@ main(int argc, char * argv[])
     if (do_help) {
         if (do_help > 1) {
             usage();
-            fprintf(stderr, "\n");
+            pr2serr("\n");
             print_modes();
         } else
             usage();
@@ -353,23 +389,27 @@ main(int argc, char * argv[])
         }
         if (optind < argc) {
             for (; optind < argc; ++optind)
-                fprintf(stderr, "Unexpected extra argument: %s\n",
-                        argv[optind]);
+                pr2serr("Unexpected extra argument: %s\n", argv[optind]);
             usage();
             return SG_LIB_SYNTAX_ERROR;
         }
     }
 
     if (NULL == device_name) {
-        fprintf(stderr, "missing device name!\n");
+        pr2serr("missing device name!\n");
         usage();
         return SG_LIB_SYNTAX_ERROR;
     }
 
+    if ((wb_len > 0) && (bpw > wb_len)) {
+        pr2serr("trim chunk size (CS) to be the same as LEN\n");
+        bpw = wb_len;
+    }
+
 #ifdef SG_LIB_WIN32
 #ifdef SG_LIB_WIN32_DIRECT
     if (verbose > 4)
-        fprintf(stderr, "Initial win32 SPT interface state: %s\n",
+        pr2serr("Initial win32 SPT interface state: %s\n",
                 scsi_pt_win32_spt_state() ? "direct" : "indirect");
     scsi_pt_win32_direct(SG_LIB_WIN32_DIRECT /* SPT pt interface */);
 #endif
@@ -377,7 +417,7 @@ main(int argc, char * argv[])
 
     sg_fd = sg_cmds_open_device(device_name, 0 /* rw */, verbose);
     if (sg_fd < 0) {
-        fprintf(stderr, ME "open error: %s: %s\n", device_name,
+        pr2serr(ME "open error: %s: %s\n", device_name,
                 safe_strerror(-sg_fd));
         return SG_LIB_FILE_ERROR;
     }
@@ -385,7 +425,7 @@ main(int argc, char * argv[])
         if (0 == wb_len)
             wb_len = DEF_XFER_LEN;
         if (NULL == (dop = (unsigned char *)malloc(wb_len))) {
-            fprintf(stderr, ME "out of memory\n");
+            pr2serr(ME "out of memory\n");
             ret = SG_LIB_SYNTAX_ERROR;
             goto err_out;
         }
@@ -394,7 +434,7 @@ main(int argc, char * argv[])
             got_stdin = (0 == strcmp(file_name, "-")) ? 1 : 0;
             if (got_stdin) {
                 if (wb_skip > 0) {
-                    fprintf(stderr, "Can't skip on stdin\n");
+                    pr2serr("Can't skip on stdin\n");
                     ret = SG_LIB_FILE_ERROR;
                     goto err_out;
                 }
@@ -431,14 +471,14 @@ main(int argc, char * argv[])
             }
             if (res < wb_len) {
                 if (wb_len_given) {
-                    fprintf(stderr, "tried to read %d bytes from %s, got "
-                            "%d bytes\n", wb_len, file_name, res);
-                    fprintf(stderr, "pad with 0xff bytes and continue\n");
+                    pr2serr("tried to read %d bytes from %s, got %d bytes\n",
+                            wb_len, file_name, res);
+                    pr2serr("pad with 0xff bytes and continue\n");
                 } else {
                     if (verbose) {
-                        fprintf(stderr, "tried to read %d bytes from %s, got "
-                                "%d bytes\n", wb_len, file_name, res);
-                        fprintf(stderr, "will write %d bytes\n", res);
+                        pr2serr("tried to read %d bytes from %s, got %d "
+                                "bytes\n", wb_len, file_name, res);
+                        pr2serr("will write %d bytes\n", res);
                     }
                     wb_len = res;
                 }
@@ -448,30 +488,42 @@ main(int argc, char * argv[])
         }
     }
 
-    res = sg_ll_write_buffer_v2(sg_fd, wb_mode, wb_mspec, wb_id, wb_offset,
-                                dop, wb_len, 1, verbose);
-    if (0 != res) {
-        ret = res;
-        switch (res) {
-        case SG_LIB_CAT_NOT_READY:
-            fprintf(stderr, "Write buffer failed, device not ready\n");
-            break;
-        case SG_LIB_CAT_UNIT_ATTENTION:
-            fprintf(stderr, "Write buffer not done, unit attention\n");
-            break;
-        case SG_LIB_CAT_ABORTED_COMMAND:
-            fprintf(stderr, "Write buffer, aborted command\n");
-            break;
-        case SG_LIB_CAT_INVALID_OP:
-            fprintf(stderr, "Write buffer command not supported\n");
-            break;
-        case SG_LIB_CAT_ILLEGAL_REQ:
-            fprintf(stderr, "bad field in Write buffer cdb\n");
-            break;
-        default:
-            fprintf(stderr, "Write buffer failed res=%d\n", res);
-            break;
+    res = 0;
+    if (bpw > 0) {
+        for (k = 0; k < wb_len; k += n) {
+            n = wb_len - k;
+            if (n > bpw)
+                n = bpw;
+            if (verbose)
+                pr2serr("sending write buffer, mode=0x%x, mspec=%d, id=%d, "
+                        " offset=%d, len=%d\n", wb_mode, wb_mspec, wb_id,
+                        wb_offset + k, n);
+            res = sg_ll_write_buffer_v2(sg_fd, wb_mode, wb_mspec, wb_id,
+                                        wb_offset + k, dop + k, n, 1,
+                                        verbose);
+            if (res)
+                break;
         }
+        if (bpw_then_activate) {
+            if (verbose)
+                pr2serr("sending Activate deferred microcode [0xf]\n");
+            res = sg_ll_write_buffer_v2(sg_fd, 0xf, 0, 0, 0, NULL, 0, 1,
+                                        verbose);
+        }
+    } else {
+        if (verbose)
+            pr2serr("sending single write buffer, mode=0x%x, mpsec=%d, "
+                    "id=%d, offset=%d, len=%d\n", wb_mode, wb_mspec, wb_id,
+                    wb_offset, wb_len);
+        res = sg_ll_write_buffer_v2(sg_fd, wb_mode, wb_mspec, wb_id,
+                                    wb_offset, dop, wb_len, 1, verbose);
+    }
+    if (0 != res) {
+        char b[80];
+
+        ret = res;
+        sg_get_category_sense_str(res, sizeof(b), b, verbose);
+        pr2serr("Write buffer failed: %s\n", b);
     }
 
 err_out:
@@ -479,7 +531,7 @@ err_out:
         free(dop);
     res = sg_cmds_close_device(sg_fd);
     if (res < 0) {
-        fprintf(stderr, "close error: %s\n", safe_strerror(-res));
+        pr2serr("close error: %s\n", safe_strerror(-res));
         if (0 == ret)
             return SG_LIB_FILE_ERROR;
     }
